<?php

namespace Drupal\telega_chatwoot\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Controller\ControllerBase;
use Drupal\telega_bot\Entity\TelegaBot;
use Drupal\telega_session\Entity\TelegaSession;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns responses for guzzle routes.
 */
class WebhookController extends ControllerBase {

  const DEBUG = TRUE;
  const S3_URL = "https://biz-red-chat.storage.yandexcloud.net/s3fs-public/";

  /**
   * Build BOT-webhook responce.
   */
  public function webhookBot(Request $request, TelegaBot $bot) {
    $chat = "";
    $payload = $request->getContent();
    $this->debug($bot, $payload);

    $data = Json::decode($payload);
    $message_type = $data['message_type'] ?? "";
    if ($message_type == 'outgoing' && !$data['private']) {
      // Сообщение оператора из чатвута отправляем в телегу юзеру.
      if ($chat = $this->chatId($data)) {
        $message = $data['content'];
        $bot_service = \Drupal::service('telega_bot')->init($bot);
        $bot_service->send($chat, $message);
      }
    }
    return new JsonResponse(["status" => "ok", "chat" => $chat]);
  }

  /**
   * Build Session-webhook responce.
   */
  public function webhookSession(Request $request, TelegaSession $session) {
    $chat = "";
    $payload = $request->getContent();
    $this->debug($session, $payload);

    $data = Json::decode($payload);
    $message_type = $data['message_type'] ?? "";
    if ($message_type == 'outgoing' && !$data['private']) {
      // Сообщение оператора из чатвута отправляем в телегу юзеру.
      if ($chat = $this->chatId($data)) {
        $message = $data['content'];
        $session_service = \Drupal::service('telega_session')->init($session);
        $session_service = $session_service->sendMessage($message);
      }
    }
    return new JsonResponse(["status" => "ok", "chat" => $chat]);
  }

  /**
   * Debug staff.
   */
  private function debug(TelegaBot|TelegaSession $entity, string $payload) : void {
    if (self::DEBUG) {
      \Drupal::logger(__CLASS__ . __FUNCTION__)->notice(
      '[@type:@bot] Payload: %pl', [
        '@type' => $entity->getEntityTypeId(),
        '@bot' => $entity->id(),
        '%pl' => $payload,
      ]);
    }
  }

  /**
   * Get chat-id from payload.
   */
  private function chatId(array $data) : string {
    $from_conversation = $data['conversation']['custom_attributes']['id'] ?? NULL;
    $from_user = $data['conversation']['meta']['sender']['custom_attributes']['id'] ?? "";
    return $from_conversation ?? $from_user;
  }

  /**
   * Хук для всех ботов типа как получится, лучше не использовать.
   */
  public function build(Request $request) {
    $payload = $request->getContent();
    $data = Json::decode($payload);
    if (empty($data)) {
      $response = new JsonResponse(["status" => "ok"]);
      return $response;
    }
    \Drupal::service('chatwoot.action')
      ->notifyAboutDeliveredWelcomeMessage($data);

    if ($data['message_type'] !== 'outgoing') {
      $response = new JsonResponse(["status" => "ok"]);
      return $response;
    }

    $bot = $data['conversation']['custom_attributes']['bot'] ?? NULL;
    if (empty($bot)) {
      $response = new JsonResponse(["status" => "ok"]);
      return $response;
    }
    $bid = str_replace('sender_', '', $bot);
    $storage = \Drupal::entityTypeManager()->getStorage('telega_bot');
    $bot_entity = $storage->load($bid);
    if (empty($bot_entity)) {
      $response = new JsonResponse(["status" => "ok"]);
      return $response;
    }
    $bot = \Drupal::service('telega_bot')->init($bot_entity);
    $chat = $data['conversation']['custom_attributes']['id'] ?? "";
    if (!$chat) {
      $chat = $data['conversation']['meta']['sender']['custom_attributes']['id'] ?? '';
    }
    $bot->send($chat, $data['content'] ?? 'empty message');

    $response = new JsonResponse(["status" => "ok"]);
    return $response;
  }

}
